unit Unit1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Grids, StdCtrls, MathEdit, ExtCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Button1: TButton;
    Label1: TLabel;
    IntEdit1: TIntEdit;
    PaintBox1: TPaintBox;
    IntEdit2: TIntEdit;
    Label2: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure IntEdit2Exit(Sender: TObject);
  private
    { Private-Deklarationen }
    erg_tab : Array[2..60] of LongInt;
                 {  2 ist die minimal mgliche Augensumme,
                   60 ist die maximal mgliche Augensumme }
    x0, y0    : Integer; { Ort des Ursprungs in PaintBox1 }
    xmin, xmax,          { Zeichenbereich in              }
    ymin, ymax,          {     User-Koordinaten           }
    ppcm      : Double;  { Pixel per Zentimeter           }
    k         : Integer; { Anzahl der Wrfel              }
    function userx(pbx: Integer): Double;
    function usery(pby: Integer): Double;
    function paintx(ux: Double): Integer;
    function painty(uy: Double): Integer;
    procedure InitScale;
    procedure ArrayLoeschen;
    function  Augensumme(wuerfel_anzahl: Integer): Integer;
    procedure ArrayFuellen;
    procedure Saeule(xlu, breite, hoehe: Double);
    procedure GrafikDarstellen;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{ =========== Interne Hilfsfunktionen ============= }

function TForm1.userx(pbx: Integer): Double;
  begin
  Result := (pbx - x0) / ppcm;
  end;

function TForm1.usery(pby: Integer): Double;
  begin
  Result := (y0 - pby) / ppcm;
  end;

function TForm1.paintx(ux: Double): Integer;
  begin
  Result := Round(ux*ppcm + x0);
  end;

function TForm1.painty(uy: Double): Integer;
  begin
  Result := Round(y0 - uy*ppcm);
  end;

procedure TForm1.InitScale;
  begin
  k    := IntEdit2.Value;
  ppcm := PaintBox1.Width / (k * 5 + 2); { Skalierung initialisieren }
  x0   := Round(ppcm/2);
  If x0 > 32 then
    y0   := PaintBox1.Height - x0
  else
    y0   := PaintBox1.Height - 32;
  { Dann stehen unter der "x-Achse" noch mindestens
    32 Pixel-Zeilen fr die Beschriftung zur Verfgung. }

  xmin := userx(0);                { Zeichenbereich ermitteln  }
  xmax := userx(PaintBox1.Width);
  ymin := usery(PaintBox1.Height);
  ymax := usery(0);
  end;

{ =========== Wrfeln ============================= }

procedure TForm1.ArrayLoeschen;
  var i : Integer;
  begin
  i := k;                        { Ergebnis-Array lschen }
  While i <= k * 6 do begin
    erg_tab[i] := 0;
    i := i + 1;
    end;
  end;

function TForm1.Augensumme(wuerfel_anzahl: Integer): Integer;
  var wz,           { Einzelne Wrfelzahl }
      i : Integer;
  begin
  Result := 0;
  i := 1;
  While i <= wuerfel_anzahl do begin
    wz  := 1 + Random(6);
    Result := Result + wz;
    i := i + 1;
    end;
  end;

procedure TForm1.ArrayFuellen;
  var asum : Integer;
      i    : LongInt;
  begin
  i := 0;
  While i < IntEdit1.Value do begin
    asum := Augensumme(k);              { Wrfeln....                  }
    erg_tab[asum] := erg_tab[asum] + 1; { ... und Ergebnisse verbuchen }
    i := i + 1;
    end;
  end;

{ =============== Grafik ========================== }

procedure TForm1.Saeule(xlu, breite, hoehe: Double);
  var oldPenCol,
      oldBrushCol : TColor;
  begin
  With PaintBox1.Canvas do begin
    oldPenCol := Pen.Color;       { alte Farben sichern }
    oldBrushCol := Brush.Color;

    Pen.Color   := clred;         { (Vorderseite der) Sule ausgeben }
    Brush.Style := bsSolid;
    Brush.Color := clLime;
    Rectangle(paintx(xlu         ), painty(0),
              paintx(xlu + breite), painty(hoehe));

    { Ab hier folgt der Luxus: }
    Brush.Color := clBlue;
    Polygon([Point(paintx(xlu + breite), painty(0)),
             Point(paintx(xlu + breite + 0.3), painty(0.3)),
             Point(paintx(xlu + breite + 0.3), painty(0.3 + hoehe)),
             Point(paintx(xlu + breite), painty(hoehe))]);
    Brush.Color := clAqua;
    Polygon([Point(paintx(xlu + breite), painty(hoehe)),
             Point(paintx(xlu + breite + 0.3), painty(hoehe + 0.3)),
             Point(paintx(xlu + 0.3), painty(hoehe + 0.3)),
             Point(paintx(xlu), painty(hoehe))]);

    Pen.Color := oldPenCol;       { alte Farben wiederherstellen }
    Brush.Color := oldBrushCol;
    end;
  end;

procedure TForm1.GrafikDarstellen;
  var i    : Integer;
      hmax : Double;
      nmax : LongInt;
  begin
  nmax := 1;        { Maximum der Ergebnis-Array-Eintrge suchen }
     { Eigentlich mte nmax auf 0 initialisiert werden, da das
       Ergebnis-Array ja in allen Feldern eine Null enthalten kann;
       da aber unten durch nmax dividiert wird, initialisieren wir
       nmax lieber mit 1, um uns dort eine komplizierte technische
       Fallunterscheidung zu ersparen. Wenn nmax eigentlich den Wert
       Null hat, dann sind ja *alle* Felder in erg_tab auch Null;
       somit werden dann auch nur Sulen der Hhe Null gezeichnet,
       was korrekt ist!  }
  i := k;
  While i <= 6 * k do begin
    If erg_tab[i] > nmax then
      nmax := erg_tab[i];
    i := i + 1;
    end;
  hmax := ymax + ymin;  { Maximale Hhe einer Sule
                          im User-Koordinaten-System }
  i := k;
  While i <= 6 * k do begin
    PaintBox1.Canvas.TextOut(paintx(i-k+0.3), painty(-0.1), IntToStr(i));
    Saeule(i-k, 0.8, erg_tab[i]*hmax/nmax);
    i := i + 1;
    end;
  end;


{ ============ Ereignis-Prozeduren ================ }

procedure TForm1.Button1Click(Sender: TObject);
  begin
  Screen.Cursor := crHourGlass;
  ArrayLoeschen;
  ArrayFuellen;
  PaintBox1.Invalidate;
  Screen.Cursor := crDefault;
  end;

procedure TForm1.FormCreate(Sender: TObject);
  begin
  InitScale;
  end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
  begin
  GrafikDarstellen;
  end;

procedure TForm1.FormResize(Sender: TObject);
  begin
  InitScale;
  end;

procedure TForm1.IntEdit2Exit(Sender: TObject);
  begin
  InitScale;
  ArrayLoeschen;
  PaintBox1.Invalidate;
  end;


{ Zum Weiterdenken:

  In der Prozedur ArrayFuellen stehen die Zeilen:

    asum := Augensumme(k);
    erg_tab[asum] := erg_tab[asum] + 1;

  Ist da die lokale Variable asum nicht eigentlich berflssig?
  Knnte man in der 2. Zeile asum nicht durch 2 Aufrufe der
  Funktion Augensumme ersetzen?

  Wenn Ihnen kein gutes Argument einfllt, das diese Frage
  entscheiden knnte, probieren Sie's doch einfach mal aus!    }

end.
